@@ -1,24 +1,44 @@
-""":mod:`pandas.io.html` is a module containing functionality for dealing with
+"""
+:mod:`pandas.io.html` is a module containing functionality for dealing with
 HTML IO.
 
 """
 
-from distutils.version import LooseVersion
-import numbers
-import os
-import re
+from __future__ import annotations
 
-import pandas.compat as compat
-from pandas.compat import (
-    binary_type, iteritems, lmap, lrange, raise_with_traceback, string_types,
-    u)
-from pandas.errors import AbstractMethodError, EmptyDataError
+from collections import abc
+import numbers
+import re
+from typing import (
+    Pattern,
+    Sequence,
+    cast,
+)
+
+from pandas._typing import (
+    FilePath,
+    ReadBuffer,
+)
+from pandas.compat._optional import import_optional_dependency
+from pandas.errors import (
+    AbstractMethodError,
+    EmptyDataError,
+)
+from pandas.util._decorators import deprecate_nonkeyword_arguments
 
 from pandas.core.dtypes.common import is_list_like
 
-from pandas import Series
+from pandas.core.construction import create_series_with_explicit_dtype
+from pandas.core.frame import DataFrame
 
-from pandas.io.common import _is_url, _validate_header_arg, urlopen
+from pandas.io.common import (
+    file_exists,
+    get_handle,
+    is_url,
+    stringify_path,
+    urlopen,
+    validate_header_arg,
+)
 from pandas.io.formats.printing import pprint_thing
 from pandas.io.parsers import TextParser
 
@@ -28,7 +48,7 @@ _HAS_LXML = False
 _HAS_HTML5LIB = False
 
 
-def _importers():
+def _importers() -> None:
     # import things we need
     # but make this done on a first use basis
 
@@ -37,24 +57,14 @@ def _importers():
         return
 
     global _HAS_BS4, _HAS_LXML, _HAS_HTML5LIB
+    bs4 = import_optional_dependency("bs4", errors="ignore")
+    _HAS_BS4 = bs4 is not None
 
-    try:
-        import bs4  # noqa
-        _HAS_BS4 = True
-    except ImportError:
-        pass
+    lxml = import_optional_dependency("lxml.etree", errors="ignore")
+    _HAS_LXML = lxml is not None
 
-    try:
-        import lxml  # noqa
-        _HAS_LXML = True
-    except ImportError:
-        pass
-
-    try:
-        import html5lib  # noqa
-        _HAS_HTML5LIB = True
-    except ImportError:
-        pass
+    html5lib = import_optional_dependency("html5lib", errors="ignore")
+    _HAS_HTML5LIB = html5lib is not None
 
     _IMPORTS = True
 
@@ -62,21 +72,18 @@ def _importers():
 #############
 # READ HTML #
 #############
-_RE_WHITESPACE = re.compile(r'[\r\n]+|\s{2,}')
+_RE_WHITESPACE = re.compile(r"[\r\n]+|\s{2,}")
 
 
-char_types = string_types + (binary_type,)
-
-
-def _remove_whitespace(s, regex=_RE_WHITESPACE):
-    """Replace extra whitespace inside of a string with a single space.
+def _remove_whitespace(s: str, regex: Pattern = _RE_WHITESPACE) -> str:
+    """
+    Replace extra whitespace inside of a string with a single space.
 
     Parameters
     ----------
     s : str or unicode
         The string from which to remove extra whitespace.
-
-    regex : regex
+    regex : re.Pattern
         The regular expression to use to remove extra whitespace.
 
     Returns
@@ -84,11 +91,12 @@ def _remove_whitespace(s, regex=_RE_WHITESPACE):
     subd : str or unicode
         `s` with all extra whitespace replaced with a single space.
     """
-    return regex.sub(' ', s.strip())
+    return regex.sub(" ", s.strip())
 
 
-def _get_skiprows(skiprows):
-    """Get an iterator given an integer, slice or container.
+def _get_skiprows(skiprows: int | Sequence[int] | slice | None) -> int | Sequence[int]:
+    """
+    Get an iterator given an integer, slice or container.
 
     Parameters
     ----------
@@ -106,46 +114,53 @@ def _get_skiprows(skiprows):
         A proper iterator to use to skip rows of a DataFrame.
     """
     if isinstance(skiprows, slice):
-        return lrange(skiprows.start or 0, skiprows.stop, skiprows.step or 1)
+        start, step = skiprows.start or 0, skiprows.step or 1
+        return list(range(start, skiprows.stop, step))
     elif isinstance(skiprows, numbers.Integral) or is_list_like(skiprows):
-        return skiprows
+        return cast("int | Sequence[int]", skiprows)
     elif skiprows is None:
         return 0
-    raise TypeError('%r is not a valid type for skipping rows' %
-                    type(skiprows).__name__)
+    raise TypeError(f"{type(skiprows).__name__} is not a valid type for skipping rows")
 
 
-def _read(obj):
-    """Try to read from a url, file or string.
+def _read(
+    obj: bytes | FilePath | ReadBuffer[str] | ReadBuffer[bytes], encoding: str | None
+) -> str | bytes:
+    """
+    Try to read from a url, file or string.
 
     Parameters
     ----------
-    obj : str, unicode, or file-like
+    obj : str, unicode, path object, or file-like object
 
     Returns
     -------
     raw_text : str
     """
-    if _is_url(obj):
-        with urlopen(obj) as url:
-            text = url.read()
-    elif hasattr(obj, 'read'):
-        text = obj.read()
-    elif isinstance(obj, char_types):
+    text: str | bytes
+    if (
+        is_url(obj)
+        or hasattr(obj, "read")
+        or (isinstance(obj, str) and file_exists(obj))
+    ):
+        # error: Argument 1 to "get_handle" has incompatible type "Union[str, bytes,
+        # Union[IO[Any], RawIOBase, BufferedIOBase, TextIOBase, TextIOWrapper, mmap]]";
+        # expected "Union[PathLike[str], Union[str, Union[IO[Any], RawIOBase,
+        # BufferedIOBase, TextIOBase, TextIOWrapper, mmap]]]"
+        with get_handle(
+            obj, "r", encoding=encoding  # type: ignore[arg-type]
+        ) as handles:
+            text = handles.handle.read()
+    elif isinstance(obj, (str, bytes)):
         text = obj
-        try:
-            if os.path.isfile(text):
-                with open(text, 'rb') as f:
-                    return f.read()
-        except (TypeError, ValueError):
-            pass
     else:
-        raise TypeError("Cannot read object of type %r" % type(obj).__name__)
+        raise TypeError(f"Cannot read object of type '{type(obj).__name__}'")
     return text
 
 
-class _HtmlFrameParser(object):
-    """Base class for parsers that parse HTML into DataFrames.
+class _HtmlFrameParser:
+    """
+    Base class for parsers that parse HTML into DataFrames.
 
     Parameters
     ----------
@@ -165,8 +180,6 @@ class _HtmlFrameParser(object):
     displayed_only : bool
         Whether or not items with "display:none" should be ignored
 
-        .. versionadded:: 0.23.0
-
     Attributes
     ----------
     io : str or file-like
@@ -185,8 +198,6 @@ class _HtmlFrameParser(object):
     displayed_only : bool
         Whether or not items with "display:none" should be ignored
 
-        .. versionadded:: 0.23.0
-
     Notes
     -----
     To subclass this class effectively you must override the following methods:
@@ -203,7 +214,14 @@ class _HtmlFrameParser(object):
     functionality.
     """
 
-    def __init__(self, io, match, attrs, encoding, displayed_only):
+    def __init__(
+        self,
+        io: FilePath | ReadBuffer[str] | ReadBuffer[bytes],
+        match: str | Pattern,
+        attrs: dict[str, str] | None,
+        encoding: str,
+        displayed_only: bool,
+    ):
         self.io = io
         self.match = match
         self.attrs = attrs
@@ -258,7 +276,8 @@ class _HtmlFrameParser(object):
         raise AbstractMethodError(self)
 
     def _parse_td(self, obj):
-        """Return the td elements from a row element.
+        """
+        Return the td elements from a row element.
 
         Parameters
         ----------
@@ -402,14 +421,12 @@ class _HtmlFrameParser(object):
                - Move rows from bottom of body to footer only if
                  all elements inside row are <th>
         """
-
         header_rows = self._parse_thead_tr(table_html)
         body_rows = self._parse_tbody_tr(table_html)
         footer_rows = self._parse_tfoot_tr(table_html)
 
         def row_is_all_th(row):
-            return all(self._equals_tag(t, 'th') for t in
-                       self._parse_td(row))
+            return all(self._equals_tag(t, "th") for t in self._parse_td(row))
 
         if not header_rows:
             # The table has no <thead>. Move the top all-<th> rows from
@@ -443,9 +460,8 @@ class _HtmlFrameParser(object):
         Any cell with ``rowspan`` or ``colspan`` will have its contents copied
         to subsequent cells.
         """
-
         all_texts = []  # list of rows, each a list of str
-        remainder = []  # list of (index, text, nrows)
+        remainder: list[tuple[int, str, int]] = []  # list of (index, text, nrows)
 
         for tr in rows:
             texts = []  # the output for this row
@@ -460,14 +476,13 @@ class _HtmlFrameParser(object):
                     prev_i, prev_text, prev_rowspan = remainder.pop(0)
                     texts.append(prev_text)
                     if prev_rowspan > 1:
-                        next_remainder.append((prev_i, prev_text,
-                                               prev_rowspan - 1))
+                        next_remainder.append((prev_i, prev_text, prev_rowspan - 1))
                     index += 1
 
                 # Append the text from this <td>, colspan times
                 text = _remove_whitespace(self._text_getter(td))
-                rowspan = int(self._attr_getter(td, 'rowspan') or 1)
-                colspan = int(self._attr_getter(td, 'colspan') or 1)
+                rowspan = int(self._attr_getter(td, "rowspan") or 1)
+                colspan = int(self._attr_getter(td, "colspan") or 1)
 
                 for _ in range(colspan):
                     texts.append(text)
@@ -479,8 +494,7 @@ class _HtmlFrameParser(object):
             for prev_i, prev_text, prev_rowspan in remainder:
                 texts.append(prev_text)
                 if prev_rowspan > 1:
-                    next_remainder.append((prev_i, prev_text,
-                                           prev_rowspan - 1))
+                    next_remainder.append((prev_i, prev_text, prev_rowspan - 1))
 
             all_texts.append(texts)
             remainder = next_remainder
@@ -493,8 +507,7 @@ class _HtmlFrameParser(object):
             for prev_i, prev_text, prev_rowspan in remainder:
                 texts.append(prev_text)
                 if prev_rowspan > 1:
-                    next_remainder.append((prev_i, prev_text,
-                                           prev_rowspan - 1))
+                    next_remainder.append((prev_i, prev_text, prev_rowspan - 1))
             all_texts.append(texts)
             remainder = next_remainder
 
@@ -519,12 +532,17 @@ class _HtmlFrameParser(object):
         if not self.displayed_only:
             return tbl_list
 
-        return [x for x in tbl_list if "display:none" not in
-                getattr(x, attr_name).get('style', '').replace(" ", "")]
+        return [
+            x
+            for x in tbl_list
+            if "display:none"
+            not in getattr(x, attr_name).get("style", "").replace(" ", "")
+        ]
 
 
 class _BeautifulSoupHtml5LibFrameParser(_HtmlFrameParser):
-    """HTML to DataFrame parser that uses BeautifulSoup under the hood.
+    """
+    HTML to DataFrame parser that uses BeautifulSoup under the hood.
 
     See Also
     --------
@@ -538,17 +556,17 @@ class _BeautifulSoupHtml5LibFrameParser(_HtmlFrameParser):
     """
 
     def __init__(self, *args, **kwargs):
-        super(_BeautifulSoupHtml5LibFrameParser, self).__init__(*args,
-                                                                **kwargs)
+        super().__init__(*args, **kwargs)
         from bs4 import SoupStrainer
-        self._strainer = SoupStrainer('table')
+
+        self._strainer = SoupStrainer("table")
 
     def _parse_tables(self, doc, match, attrs):
         element_name = self._strainer.name
         tables = doc.find_all(element_name, attrs=attrs)
 
         if not tables:
-            raise ValueError('No tables found')
+            raise ValueError("No tables found")
 
         result = []
         unique_tables = set()
@@ -556,18 +574,15 @@ class _BeautifulSoupHtml5LibFrameParser(_HtmlFrameParser):
 
         for table in tables:
             if self.displayed_only:
-                for elem in table.find_all(
-                        style=re.compile(r"display:\s*none")):
+                for elem in table.find_all(style=re.compile(r"display:\s*none")):
                     elem.decompose()
 
-            if (table not in unique_tables and
-                    table.find(text=match) is not None):
+            if table not in unique_tables and table.find(text=match) is not None:
                 result.append(table)
             unique_tables.add(table)
 
         if not result:
-            raise ValueError("No tables found matching pattern {patt!r}"
-                             .format(patt=match.pattern))
+            raise ValueError(f"No tables found matching pattern {repr(match.pattern)}")
         return result
 
     def _text_getter(self, obj):
@@ -577,35 +592,42 @@ class _BeautifulSoupHtml5LibFrameParser(_HtmlFrameParser):
         return obj.name == tag
 
     def _parse_td(self, row):
-        return row.find_all(('td', 'th'), recursive=False)
+        return row.find_all(("td", "th"), recursive=False)
 
     def _parse_thead_tr(self, table):
-        return table.select('thead tr')
+        return table.select("thead tr")
 
     def _parse_tbody_tr(self, table):
-        from_tbody = table.select('tbody tr')
-        from_root = table.find_all('tr', recursive=False)
+        from_tbody = table.select("tbody tr")
+        from_root = table.find_all("tr", recursive=False)
         # HTML spec: at most one of these lists has content
         return from_tbody + from_root
 
     def _parse_tfoot_tr(self, table):
-        return table.select('tfoot tr')
+        return table.select("tfoot tr")
 
     def _setup_build_doc(self):
-        raw_text = _read(self.io)
+        raw_text = _read(self.io, self.encoding)
         if not raw_text:
-            raise ValueError('No text parsed from document: {doc}'
-                             .format(doc=self.io))
+            raise ValueError(f"No text parsed from document: {self.io}")
         return raw_text
 
     def _build_doc(self):
         from bs4 import BeautifulSoup
-        return BeautifulSoup(self._setup_build_doc(), features='html5lib',
-                             from_encoding=self.encoding)
+
+        bdoc = self._setup_build_doc()
+        if isinstance(bdoc, bytes) and self.encoding is not None:
+            udoc = bdoc.decode(self.encoding)
+            from_encoding = None
+        else:
+            udoc = bdoc
+            from_encoding = self.encoding
+        return BeautifulSoup(udoc, features="html5lib", from_encoding=from_encoding)
 
 
-def _build_xpath_expr(attrs):
-    """Build an xpath expression to simulate bs4's ability to pass in kwargs to
+def _build_xpath_expr(attrs) -> str:
+    """
+    Build an xpath expression to simulate bs4's ability to pass in kwargs to
     search for attributes when using the lxml parser.
 
     Parameters
@@ -619,19 +641,19 @@ def _build_xpath_expr(attrs):
         An XPath expression that checks for the given HTML attributes.
     """
     # give class attribute as class_ because class is a python keyword
-    if 'class_' in attrs:
-        attrs['class'] = attrs.pop('class_')
+    if "class_" in attrs:
+        attrs["class"] = attrs.pop("class_")
 
-    s = [u("@{key}={val!r}").format(key=k, val=v) for k, v in iteritems(attrs)]
-    return u('[{expr}]').format(expr=' and '.join(s))
+    s = " and ".join([f"@{k}={repr(v)}" for k, v in attrs.items()])
+    return f"[{s}]"
 
 
-_re_namespace = {'re': 'http://exslt.org/regular-expressions'}
-_valid_schemes = 'http', 'file', 'ftp'
+_re_namespace = {"re": "http://exslt.org/regular-expressions"}
 
 
 class _LxmlFrameParser(_HtmlFrameParser):
-    """HTML to DataFrame parser that uses lxml under the hood.
+    """
+    HTML to DataFrame parser that uses lxml under the hood.
 
     Warning
     -------
@@ -648,24 +670,20 @@ class _LxmlFrameParser(_HtmlFrameParser):
     :class:`_HtmlFrameParser`.
     """
 
-    def __init__(self, *args, **kwargs):
-        super(_LxmlFrameParser, self).__init__(*args, **kwargs)
-
     def _text_getter(self, obj):
         return obj.text_content()
 
     def _parse_td(self, row):
         # Look for direct children only: the "row" element here may be a
         # <thead> or <tfoot> (see _parse_thead_tr).
-        return row.xpath('./td|./th')
+        return row.xpath("./td|./th")
 
     def _parse_tables(self, doc, match, kwargs):
         pattern = match.pattern
 
         # 1. check all descendants for the given pattern and only search tables
         # 2. go up the tree until we find a table
-        query = '//table//*[re:test(text(), {patt!r})]/ancestor::table'
-        xpath_expr = u(query).format(patt=pattern)
+        xpath_expr = f"//table//*[re:test(text(), {repr(pattern)})]/ancestor::table"
 
         # if any table attributes were given build an xpath expression to
         # search for them
@@ -680,14 +698,12 @@ class _LxmlFrameParser(_HtmlFrameParser):
                 # lxml utilizes XPATH 1.0 which does not have regex
                 # support. As a result, we find all elements with a style
                 # attribute and iterate them to check for display:none
-                for elem in table.xpath('.//*[@style]'):
-                    if "display:none" in elem.attrib.get(
-                            "style", "").replace(" ", ""):
+                for elem in table.xpath(".//*[@style]"):
+                    if "display:none" in elem.attrib.get("style", "").replace(" ", ""):
                         elem.getparent().remove(elem)
 
         if not tables:
-            raise ValueError("No tables found matching regex {patt!r}"
-                             .format(patt=pattern))
+            raise ValueError(f"No tables found matching regex {repr(pattern)}")
         return tables
 
     def _equals_tag(self, obj, tag):
@@ -709,12 +725,17 @@ class _LxmlFrameParser(_HtmlFrameParser):
         --------
         pandas.io.html._HtmlFrameParser._build_doc
         """
-        from lxml.html import parse, fromstring, HTMLParser
         from lxml.etree import XMLSyntaxError
+        from lxml.html import (
+            HTMLParser,
+            fromstring,
+            parse,
+        )
+
         parser = HTMLParser(recover=True, encoding=self.encoding)
 
         try:
-            if _is_url(self.io):
+            if is_url(self.io):
                 with urlopen(self.io) as f:
                     r = parse(f, parser=parser)
             else:
@@ -724,9 +745,9 @@ class _LxmlFrameParser(_HtmlFrameParser):
                 r = r.getroot()
             except AttributeError:
                 pass
-        except (UnicodeDecodeError, IOError) as e:
+        except (UnicodeDecodeError, OSError) as e:
             # if the input is a blob of html goop
-            if not _is_url(self.io):
+            if not is_url(self.io):
                 r = fromstring(self.io, parser=parser)
 
                 try:
@@ -736,15 +757,15 @@ class _LxmlFrameParser(_HtmlFrameParser):
             else:
                 raise e
         else:
-            if not hasattr(r, 'text_content'):
+            if not hasattr(r, "text_content"):
                 raise XMLSyntaxError("no text parsed from document", 0, 0, 0)
         return r
 
     def _parse_thead_tr(self, table):
         rows = []
 
-        for thead in table.xpath('.//thead'):
-            rows.extend(thead.xpath('./tr'))
+        for thead in table.xpath(".//thead"):
+            rows.extend(thead.xpath("./tr"))
 
             # HACK: lxml does not clean up the clearly-erroneous
             # <thead><th>foo</th><th>bar</th></thead>. (Missing <tr>). Add
@@ -752,36 +773,37 @@ class _LxmlFrameParser(_HtmlFrameParser):
             # children as though it's a <tr>.
             #
             # Better solution would be to use html5lib.
-            elements_at_root = thead.xpath('./td|./th')
+            elements_at_root = thead.xpath("./td|./th")
             if elements_at_root:
                 rows.append(thead)
 
         return rows
 
     def _parse_tbody_tr(self, table):
-        from_tbody = table.xpath('.//tbody//tr')
-        from_root = table.xpath('./tr')
+        from_tbody = table.xpath(".//tbody//tr")
+        from_root = table.xpath("./tr")
         # HTML spec: at most one of these lists has content
         return from_tbody + from_root
 
     def _parse_tfoot_tr(self, table):
-        return table.xpath('.//tfoot//tr')
+        return table.xpath(".//tfoot//tr")
 
 
 def _expand_elements(body):
-    lens = Series(lmap(len, body))
+    data = [len(elem) for elem in body]
+    lens = create_series_with_explicit_dtype(data, dtype_if_empty=object)
     lens_max = lens.max()
     not_max = lens[lens != lens_max]
 
-    empty = ['']
-    for ind, length in iteritems(not_max):
+    empty = [""]
+    for ind, length in not_max.items():
         body[ind] += empty * (lens_max - length)
 
 
 def _data_to_frame(**kwargs):
-    head, body, foot = kwargs.pop('data')
-    header = kwargs.pop('header')
-    kwargs['skiprows'] = _get_skiprows(kwargs['skiprows'])
+    head, body, foot = kwargs.pop("data")
+    header = kwargs.pop("header")
+    kwargs["skiprows"] = _get_skiprows(kwargs["skiprows"])
     if head:
         body = head + body
 
@@ -791,26 +813,28 @@ def _data_to_frame(**kwargs):
                 header = 0
             else:
                 # ignore all-empty-text rows
-                header = [i for i, row in enumerate(head)
-                          if any(text for text in row)]
+                header = [i for i, row in enumerate(head) if any(text for text in row)]
 
     if foot:
         body += foot
 
     # fill out elements of body that are "ragged"
     _expand_elements(body)
-    tp = TextParser(body, header=header, **kwargs)
-    df = tp.read()
-    return df
+    with TextParser(body, header=header, **kwargs) as tp:
+        return tp.read()
 
 
-_valid_parsers = {'lxml': _LxmlFrameParser, None: _LxmlFrameParser,
-                  'html5lib': _BeautifulSoupHtml5LibFrameParser,
-                  'bs4': _BeautifulSoupHtml5LibFrameParser}
+_valid_parsers = {
+    "lxml": _LxmlFrameParser,
+    None: _LxmlFrameParser,
+    "html5lib": _BeautifulSoupHtml5LibFrameParser,
+    "bs4": _BeautifulSoupHtml5LibFrameParser,
+}
 
 
-def _parser_dispatch(flavor):
-    """Choose the parser based on the input flavor.
+def _parser_dispatch(flavor: str | None) -> type[_HtmlFrameParser]:
+    """
+    Choose the parser based on the input flavor.
 
     Parameters
     ----------
@@ -831,20 +855,17 @@ def _parser_dispatch(flavor):
     """
     valid_parsers = list(_valid_parsers.keys())
     if flavor not in valid_parsers:
-        raise ValueError('{invalid!r} is not a valid flavor, valid flavors '
-                         'are {valid}'
-                         .format(invalid=flavor, valid=valid_parsers))
+        raise ValueError(
+            f"{repr(flavor)} is not a valid flavor, valid flavors are {valid_parsers}"
+        )
 
-    if flavor in ('bs4', 'html5lib'):
+    if flavor in ("bs4", "html5lib"):
         if not _HAS_HTML5LIB:
             raise ImportError("html5lib not found, please install it")
         if not _HAS_BS4:
-            raise ImportError(
-                "BeautifulSoup4 (bs4) not found, please install it")
-        import bs4
-        if LooseVersion(bs4.__version__) <= LooseVersion('4.2.0'):
-            raise ValueError("A minimum version of BeautifulSoup 4.2.1 "
-                             "is required")
+            raise ImportError("BeautifulSoup4 (bs4) not found, please install it")
+        # Although we call this above, we want to raise here right before use.
+        bs4 = import_optional_dependency("bs4")  # noqa:F841
 
     else:
         if not _HAS_LXML:
@@ -852,35 +873,36 @@ def _parser_dispatch(flavor):
     return _valid_parsers[flavor]
 
 
-def _print_as_set(s):
-    return ('{' + '{arg}'.format(arg=', '.join(
-        pprint_thing(el) for el in s)) + '}')
+def _print_as_set(s) -> str:
+    arg = ", ".join([pprint_thing(el) for el in s])
+    return f"{{{arg}}}"
 
 
 def _validate_flavor(flavor):
     if flavor is None:
-        flavor = 'lxml', 'bs4'
-    elif isinstance(flavor, string_types):
-        flavor = flavor,
-    elif isinstance(flavor, compat.Iterable):
-        if not all(isinstance(flav, string_types) for flav in flavor):
-            raise TypeError('Object of type {typ!r} is not an iterable of '
-                            'strings'
-                            .format(typ=type(flavor).__name__))
+        flavor = "lxml", "bs4"
+    elif isinstance(flavor, str):
+        flavor = (flavor,)
+    elif isinstance(flavor, abc.Iterable):
+        if not all(isinstance(flav, str) for flav in flavor):
+            raise TypeError(
+                f"Object of type {repr(type(flavor).__name__)} "
+                f"is not an iterable of strings"
+            )
     else:
-        fmt = '{flavor!r}' if isinstance(flavor, string_types) else '{flavor}'
-        fmt += ' is not a valid flavor'
-        raise ValueError(fmt.format(flavor=flavor))
+        msg = repr(flavor) if isinstance(flavor, str) else str(flavor)
+        msg += " is not a valid flavor"
+        raise ValueError(msg)
 
     flavor = tuple(flavor)
     valid_flavors = set(_valid_parsers)
     flavor_set = set(flavor)
 
     if not flavor_set & valid_flavors:
-        raise ValueError('{invalid} is not a valid set of flavors, valid '
-                         'flavors are {valid}'
-                         .format(invalid=_print_as_set(flavor_set),
-                                 valid=_print_as_set(valid_flavors)))
+        raise ValueError(
+            f"{_print_as_set(flavor_set)} is not a valid set of flavors, valid "
+            f"flavors are {_print_as_set(valid_flavors)}"
+        )
     return flavor
 
 
@@ -888,7 +910,6 @@ def _parse(flavor, io, match, attrs, encoding, displayed_only, **kwargs):
     flavor = _validate_flavor(flavor)
     compiled_match = re.compile(match)  # you can pass a compiled regex here
 
-    # hack around python 3 deleting the exception variable
     retained = None
     for flav in flavor:
         parser = _parser_dispatch(flav)
@@ -896,24 +917,26 @@ def _parse(flavor, io, match, attrs, encoding, displayed_only, **kwargs):
 
         try:
             tables = p.parse_tables()
-        except Exception as caught:
+        except ValueError as caught:
             # if `io` is an io-like object, check if it's seekable
             # and try to rewind it before trying the next parser
-            if hasattr(io, 'seekable') and io.seekable():
+            if hasattr(io, "seekable") and io.seekable():
                 io.seek(0)
-            elif hasattr(io, 'seekable') and not io.seekable():
+            elif hasattr(io, "seekable") and not io.seekable():
                 # if we couldn't rewind it, let the user know
-                raise ValueError('The flavor {} failed to parse your input. '
-                                 'Since you passed a non-rewindable file '
-                                 'object, we can\'t rewind it to try '
-                                 'another parser. Try read_html() with a '
-                                 'different flavor.'.format(flav))
+                raise ValueError(
+                    f"The flavor {flav} failed to parse your input. "
+                    "Since you passed a non-rewindable file "
+                    "object, we can't rewind it to try "
+                    "another parser. Try read_html() with a different flavor."
+                ) from caught
 
             retained = caught
         else:
             break
     else:
-        raise_with_traceback(retained)
+        assert retained is not None  # for mypy
+        raise retained
 
     ret = []
     for table in tables:
@@ -924,17 +947,33 @@ def _parse(flavor, io, match, attrs, encoding, displayed_only, **kwargs):
     return ret
 
 
-def read_html(io, match='.+', flavor=None, header=None, index_col=None,
-              skiprows=None, attrs=None, parse_dates=False,
-              tupleize_cols=None, thousands=',', encoding=None,
-              decimal='.', converters=None, na_values=None,
-              keep_default_na=True, displayed_only=True):
-    r"""Read HTML tables into a ``list`` of ``DataFrame`` objects.
+@deprecate_nonkeyword_arguments(version="2.0")
+def read_html(
+    io: FilePath | ReadBuffer[str],
+    match: str | Pattern = ".+",
+    flavor: str | None = None,
+    header: int | Sequence[int] | None = None,
+    index_col: int | Sequence[int] | None = None,
+    skiprows: int | Sequence[int] | slice | None = None,
+    attrs: dict[str, str] | None = None,
+    parse_dates: bool = False,
+    thousands: str | None = ",",
+    encoding: str | None = None,
+    decimal: str = ".",
+    converters: dict | None = None,
+    na_values=None,
+    keep_default_na: bool = True,
+    displayed_only: bool = True,
+) -> list[DataFrame]:
+    r"""
+    Read HTML tables into a ``list`` of ``DataFrame`` objects.
 
     Parameters
     ----------
-    io : str or file-like
-        A URL, a file-like object, or a raw string containing HTML. Note that
+    io : str, path object, or file-like object
+        String, path object (implementing ``os.PathLike[str]``), or file-like
+        object implementing a string ``read()`` function.
+        The string can represent a URL or the HTML itself. Note that
         lxml only accepts the http, ftp and file url protocols. If you have a
         URL that starts with ``'https'`` you might try removing the ``'s'``.
 
@@ -946,26 +985,26 @@ def read_html(io, match='.+', flavor=None, header=None, index_col=None,
         This value is converted to a regular expression so that there is
         consistent behavior between Beautiful Soup and lxml.
 
-    flavor : str or None, container of strings
+    flavor : str, optional
         The parsing engine to use. 'bs4' and 'html5lib' are synonymous with
         each other, they are both there for backwards compatibility. The
         default of ``None`` tries to use ``lxml`` to parse and if that fails it
         falls back on ``bs4`` + ``html5lib``.
 
-    header : int or list-like or None, optional
+    header : int or list-like, optional
         The row (or list of rows for a :class:`~pandas.MultiIndex`) to use to
         make the columns headers.
 
-    index_col : int or list-like or None, optional
+    index_col : int or list-like, optional
         The column (or list of columns) to use to create the index.
 
-    skiprows : int or list-like or slice or None, optional
-        0-based. Number of rows to skip after parsing the column integer. If a
+    skiprows : int, list-like or slice, optional
+        Number of rows to skip after parsing the column integer. 0-based. If a
         sequence of integers or a slice is given, will skip the rows indexed by
         that sequence.  Note that a single element sequence means 'skip the nth
         row' whereas an integer means 'skip n rows'.
 
-    attrs : dict or None, optional
+    attrs : dict, optional
         This is a dictionary of attributes that you can pass to use to identify
         the table in the HTML. These are not checked for validity before being
         passed to lxml or Beautiful Soup. However, these attributes must be
@@ -975,7 +1014,7 @@ def read_html(io, match='.+', flavor=None, header=None, index_col=None,
 
         is a valid attribute dictionary because the 'id' HTML tag attribute is
         a valid HTML attribute for *any* HTML tag as per `this document
-        <http://www.w3.org/TR/html-markup/global-attributes.html>`__. ::
+        <https://html.spec.whatwg.org/multipage/dom.html#global-attributes>`__. ::
 
             attrs = {'asdf': 'table'}
 
@@ -984,24 +1023,16 @@ def read_html(io, match='.+', flavor=None, header=None, index_col=None,
         table attributes can be found `here
         <http://www.w3.org/TR/REC-html40/struct/tables.html#h-11.2>`__. A
         working draft of the HTML 5 spec can be found `here
-        <http://www.w3.org/TR/html-markup/table.html>`__. It contains the
+        <https://html.spec.whatwg.org/multipage/tables.html>`__. It contains the
         latest information on table attributes for the modern web.
 
     parse_dates : bool, optional
-        See :func:`~pandas.read_csv` for more details.
-
-    tupleize_cols : bool, optional
-        If ``False`` try to parse multiple header rows into a
-        :class:`~pandas.MultiIndex`, otherwise return raw tuples. Defaults to
-        ``False``.
-
-        .. deprecated:: 0.21.0
-           This argument will be removed and will always convert to MultiIndex
+        See :func:`~read_csv` for more details.
 
     thousands : str, optional
         Separator to use to parse thousands. Defaults to ``','``.
 
-    encoding : str or None, optional
+    encoding : str, optional
         The encoding used to decode the web page. Defaults to ``None``.``None``
         preserves the previous encoding behavior, which depends on the
         underlying parser library (e.g., the parser library will try to use
@@ -1011,39 +1042,30 @@ def read_html(io, match='.+', flavor=None, header=None, index_col=None,
         Character to recognize as decimal point (e.g. use ',' for European
         data).
 
-        .. versionadded:: 0.19.0
-
     converters : dict, default None
         Dict of functions for converting values in certain columns. Keys can
         either be integers or column labels, values are functions that take one
         input argument, the cell (not column) content, and return the
         transformed content.
 
-        .. versionadded:: 0.19.0
-
     na_values : iterable, default None
-        Custom NA values
-
-        .. versionadded:: 0.19.0
+        Custom NA values.
 
     keep_default_na : bool, default True
         If na_values are specified and keep_default_na is False the default NaN
-        values are overridden, otherwise they're appended to
-
-        .. versionadded:: 0.19.0
+        values are overridden, otherwise they're appended to.
 
     displayed_only : bool, default True
-        Whether elements with "display: none" should be parsed
-
-        .. versionadded:: 0.23.0
+        Whether elements with "display: none" should be parsed.
 
     Returns
     -------
-    dfs : list of DataFrames
+    dfs
+        A list of DataFrames.
 
     See Also
     --------
-    pandas.read_csv
+    read_csv : Read a comma-separated values (csv) file into DataFrame.
 
     Notes
     -----
@@ -1064,9 +1086,7 @@ def read_html(io, match='.+', flavor=None, header=None, index_col=None,
     the header, otherwise the function attempts to find the header within
     the body (by putting rows with only ``<th>`` elements into the header).
 
-        .. versionadded:: 0.21.0
-
-    Similar to :func:`~pandas.read_csv` the `header` argument is applied
+    Similar to :func:`~read_csv` the `header` argument is applied
     **after** `skiprows` is applied.
 
     This function will *always* return a list of :class:`DataFrame` *or*
@@ -1082,13 +1102,28 @@ def read_html(io, match='.+', flavor=None, header=None, index_col=None,
     # Type check here. We don't want to parse only to fail because of an
     # invalid value of an integer skiprows.
     if isinstance(skiprows, numbers.Integral) and skiprows < 0:
-        raise ValueError('cannot skip rows starting from the end of the '
-                         'data (you passed a negative value)')
-    _validate_header_arg(header)
-    return _parse(flavor=flavor, io=io, match=match, header=header,
-                  index_col=index_col, skiprows=skiprows,
-                  parse_dates=parse_dates, tupleize_cols=tupleize_cols,
-                  thousands=thousands, attrs=attrs, encoding=encoding,
-                  decimal=decimal, converters=converters, na_values=na_values,
-                  keep_default_na=keep_default_na,
-                  displayed_only=displayed_only)
+        raise ValueError(
+            "cannot skip rows starting from the end of the "
+            "data (you passed a negative value)"
+        )
+    validate_header_arg(header)
+
+    io = stringify_path(io)
+
+    return _parse(
+        flavor=flavor,
+        io=io,
+        match=match,
+        header=header,
+        index_col=index_col,
+        skiprows=skiprows,
+        parse_dates=parse_dates,
+        thousands=thousands,
+        attrs=attrs,
+        encoding=encoding,
+        decimal=decimal,
+        converters=converters,
+        na_values=na_values,
+        keep_default_na=keep_default_na,
+        displayed_only=displayed_only,
+    )
