@@ -12,7 +12,6 @@ import pandas.util._test_decorators as td
 import pandas as pd
 from pandas import (
     DataFrame,
-    MultiIndex,
     Series,
     _testing as tm,
     concat,
@@ -25,7 +24,7 @@ from pandas.tests.io.pytables.common import (
     ensure_clean_store,
 )
 
-pytestmark = pytest.mark.single
+pytestmark = pytest.mark.single_cpu
 
 
 @pytest.mark.filterwarnings("ignore:object name:tables.exceptions.NaturalNameWarning")
@@ -78,10 +77,10 @@ def test_append(setup_path):
                         np.random.randint(0, high=65535, size=5), dtype=np.uint16
                     ),
                     "u32": Series(
-                        np.random.randint(0, high=2 ** 30, size=5), dtype=np.uint32
+                        np.random.randint(0, high=2**30, size=5), dtype=np.uint32
                     ),
                     "u64": Series(
-                        [2 ** 58, 2 ** 59, 2 ** 60, 2 ** 61, 2 ** 62],
+                        [2**58, 2**59, 2**60, 2**61, 2**62],
                         dtype=np.uint64,
                     ),
                 },
@@ -638,13 +637,9 @@ def test_append_with_data_columns(setup_path):
         tm.assert_frame_equal(result, expected)
 
 
-def test_append_hierarchical(setup_path):
-    index = MultiIndex(
-        levels=[["foo", "bar", "baz", "qux"], ["one", "two", "three"]],
-        codes=[[0, 0, 0, 1, 1, 2, 2, 3, 3, 3], [0, 1, 2, 0, 1, 1, 2, 0, 1, 2]],
-        names=["foo", "bar"],
-    )
-    df = DataFrame(np.random.randn(10, 3), index=index, columns=["A", "B", "C"])
+def test_append_hierarchical(setup_path, multiindex_dataframe_random_data):
+    df = multiindex_dataframe_random_data
+    df.columns.name = None
 
     with ensure_clean_store(setup_path) as store:
         store.append("mi", df)
@@ -675,14 +670,10 @@ def test_append_misc(setup_path):
         result = store.select("df1")
         tm.assert_frame_equal(result, df)
 
-    # more chunksize in append tests
-    def check(obj, comparator):
-        for c in [10, 200, 1000]:
-            with ensure_clean_store(setup_path, mode="w") as store:
-                store.append("obj", obj, chunksize=c)
-                result = store.select("obj")
-                comparator(result, obj)
 
+@pytest.mark.parametrize("chunksize", [10, 200, 1000])
+def test_append_misc_chunksize(setup_path, chunksize):
+    # more chunksize in append tests
     df = tm.makeDataFrame()
     df["string"] = "foo"
     df["float322"] = 1.0
@@ -690,8 +681,13 @@ def test_append_misc(setup_path):
     df["bool"] = df["float322"] > 0
     df["time1"] = Timestamp("20130101")
     df["time2"] = Timestamp("20130102")
-    check(df, tm.assert_frame_equal)
+    with ensure_clean_store(setup_path, mode="w") as store:
+        store.append("obj", df, chunksize=chunksize)
+        result = store.select("obj")
+        tm.assert_frame_equal(result, df)
 
+
+def test_append_misc_empty_frame(setup_path):
     # empty frame, GH4273
     with ensure_clean_store(setup_path) as store:
 
@@ -896,9 +892,6 @@ def test_append_to_multiple_dropna(setup_path):
         tm.assert_index_equal(store.select("df1").index, store.select("df2").index)
 
 
-@pytest.mark.xfail(
-    run=False, reason="append_to_multiple_dropna_false is not raising as failed"
-)
 def test_append_to_multiple_dropna_false(setup_path):
     df1 = tm.makeTimeDataFrame()
     df2 = tm.makeTimeDataFrame().rename(columns="{}_2".format)
@@ -912,8 +905,7 @@ def test_append_to_multiple_dropna_false(setup_path):
             {"df1a": ["A", "B"], "df2a": None}, df, selector="df1a", dropna=False
         )
 
-        # TODO Update error message to desired message for this case
-        msg = "Cannot select as multiple after appending with dropna=False"
+        msg = "all tables must have exactly the same nrows!"
         with pytest.raises(ValueError, match=msg):
             store.select_as_multiple(["df1a", "df2a"])
 
